@@ -2,7 +2,7 @@ import mmap
 import io
 
 from .exceptions import ValidationError
-from .lib import nvl
+from .lib import nvl, eytzinger_sort
 from . import compression
 
 import crc32c
@@ -53,7 +53,7 @@ class MapBuffer:
       self.buffer = self.dict2buf(data, compress)
     elif isinstance(data, io.IOBase):
       self.buffer = mmap.mmap(data.fileno(), 0, access=mmap.ACCESS_READ)
-    elif isinstance(data, (bytes, mmap.mmap)):
+    elif isinstance(data, (bytes, bytearray, mmap.mmap)):
       self.buffer = data
     elif hasattr(data, "__getitem__"):
       self.buffer = data
@@ -155,6 +155,39 @@ class MapBuffer:
 
     return value  
 
+  def setindex(self, i, data):
+    index = self.index()
+    N = index.shape[0]
+    offset = index[i,1]
+    if i < N - 1:
+      next_offset = index[i+1,1]
+      existing_length = int(next_offset - offset) 
+    else:
+      existing_length = int(len(self.buffer) - offset)
+      next_offset = int(len(self.buffer))
+
+    if self.tobytesfn:
+      data = self.tobytesfn(data)
+
+    if self.compress:
+      data = compression.compress(data, method=self.compress) 
+
+    check_length = len(data)
+    if self.format_version == 1:
+      check_length += 4
+
+    if check_length != existing_length:
+      raise ValueError(
+        f"Can only overwrite data of exactly the same length. "
+        f"Expected: {existing_length} bytes, Got: {check_length} bytes"
+      )
+
+    if self.format_version == 1:
+      self.buffer[offset:next_offset] = data
+    else:
+      data += crc32c.crc32c(data).to_bytes(4, byteorder='little')
+      self.buffer[offset:next_offset] = data
+
   def find_index_position(self, label):
     index = self.index()
     N = len(index)
@@ -191,14 +224,23 @@ class MapBuffer:
     else:
       raise KeyError("{} was not found.".format(label))
 
+  def __setitem__(self, label, label_data):
+    pos = self.find_index_position(label)
+    if pos is not None:
+      return self.setindex(pos, label_data)
+    else:
+      raise KeyError("{} was not found.".format(label))    
+
   def dict2buf(self, data, compress=None, tobytesfn=None):
     """Structure [ index length, sorted index, data ]"""
-    labels = np.array([ int(lbl) for lbl in data.keys() ], dtype=self.dtype)
+    labels = np.fromiter(
+      ( int(lbl) for lbl in data.keys() ), 
+      count=len(data), dtype=self.dtype
+    )
     labels.sort()
 
-    out = np.zeros((len(labels),), dtype=np.uint64)
-    eytzinger_sort(labels, out)
-    labels = out
+    layout = mapbufferaccel.eytzinger_sort_indices(len(data))
+    labels = labels[layout]
 
     N = len(labels)
     N_region = N.to_bytes(4, byteorder="little", signed=False)
@@ -236,6 +278,8 @@ class MapBuffer:
     for i, label in zip(range(1, len(labels)), labels):
       index[i*2 + 1] = index[(i-1)*2 + 1] + len(bytes_data[labels[i-1]])
 
+    del labels
+    
     return b"".join([ header, index.tobytes(), data_region ])
 
   def todict(self):
@@ -284,20 +328,3 @@ class MapBuffer:
   def validate_buffer(buf):
     mapbuf = MapBuffer(buf)
     return mapbuf.validate()
-
-# TODO: rewrite as a stack to prevent possible stackoverflows
-def eytzinger_sort(inpt, output, i = 0, k = 1):
-  """
-  Takes an ascendingly sorted input and 
-  an equal sized output buffer into which to 
-  rewrite the input in eytzinger order.
-
-  Modified from:
-  https://algorithmica.org/en/eytzinger
-  """
-  if k <= len(inpt):
-    i = eytzinger_sort(inpt, output, i, 2 * k)
-    output[k - 1] = inpt[i]
-    i += 1
-    i = eytzinger_sort(inpt, output,i, 2 * k + 1)
-  return i
